@@ -604,7 +604,7 @@ References
 #
 # Distributed under the terms of the Modified BSD License.
 #
-# The full license is in the file COPYING.txt, distributed with this software.
+# The full license is in the file LICENSE.txt, distributed with this software.
 # ----------------------------------------------------------------------------
 
 import itertools
@@ -778,7 +778,7 @@ def _generator_to_fasta(obj, fh, qual=FileSentinel,
         fh.write('>%s\n%s\n' % (header, seq_str))
 
         if qual is not None:
-            qual_str = ' '.join(np.asarray(qual_scores, dtype=np.str))
+            qual_str = ' '.join(np.asarray(qual_scores, dtype=str))
             if max_width is not None:
                 qual_str = qual_wrapper.fill(qual_str)
             qual.write('>%s\n%s\n' % (header, qual_str))
