@@ -107,6 +107,8 @@ import pandas as pd
 import scipy.stats
 import skbio.util
 from skbio.util._decorator import experimental
+from skbio.stats.distance import DistanceMatrix
+from scipy.sparse import coo_matrix
 
 
 @experimental(as_of="0.4.0")
@@ -229,8 +231,7 @@ def multiplicative_replacement(mat, delta=None):
 
 @experimental(as_of="0.4.0")
 def perturb(x, y):
-    r"""
-    Performs the perturbation operation.
+    r""" Performs the perturbation operation.
 
     This operation is defined as
 
@@ -279,8 +280,7 @@ def perturb(x, y):
 
 @experimental(as_of="0.4.0")
 def perturb_inv(x, y):
-    r"""
-    Performs the inverse perturbation operation.
+    r""" Performs the inverse perturbation operation.
 
     This operation is defined as
 
@@ -329,8 +329,7 @@ def perturb_inv(x, y):
 
 @experimental(as_of="0.4.0")
 def power(x, a):
-    r"""
-    Performs the power operation.
+    r""" Performs the power operation.
 
     This operation is defined as follows
 
@@ -376,8 +375,7 @@ def power(x, a):
 
 @experimental(as_of="0.4.0")
 def inner(x, y):
-    r"""
-    Calculates the Aitchson inner product.
+    r""" Calculates the Aitchson inner product.
 
     This inner product is defined as follows
 
@@ -419,8 +417,7 @@ def inner(x, y):
 
 @experimental(as_of="0.4.0")
 def clr(mat):
-    r"""
-    Performs centre log ratio transformation.
+    r""" Performs centre log ratio transformation.
 
     This function transforms compositions from Aitchison geometry to
     the real space. The :math:`clr` transform is both an isometry and an
@@ -468,8 +465,7 @@ def clr(mat):
 
 @experimental(as_of="0.4.0")
 def clr_inv(mat):
-    r"""
-    Performs inverse centre log ratio transformation.
+    r""" Performs inverse centre log ratio transformation.
 
     This function transforms compositions from the real space to
     Aitchison geometry. The :math:`clr^{-1}` transform is both an isometry,
@@ -504,15 +500,16 @@ def clr_inv(mat):
     >>> x = np.array([.1, .3, .4, .2])
     >>> clr_inv(x)
     array([ 0.21383822,  0.26118259,  0.28865141,  0.23632778])
-
     """
-    return closure(np.exp(mat))
+    # for numerical stability (aka softmax trick)
+    mat = np.atleast_2d(mat)
+    emat = np.exp(mat - mat.max(axis=-1, keepdims=True))
+    return closure(emat)
 
 
 @experimental(as_of="0.4.0")
 def ilr(mat, basis=None, check=True):
-    r"""
-    Performs isometric log ratio transformation.
+    r""" Performs isometric log ratio transformation.
 
     This function transforms compositions from Aitchison simplex to
     the real space. The :math: ilr` transform is both an isometry,
@@ -539,7 +536,7 @@ def ilr(mat, basis=None, check=True):
        rows = compositions and
        columns = components
 
-    basis: numpy.ndarray, float, optional
+    basis: numpy.ndarray or scipy.sparse.matrix, optional
         orthonormal basis for Aitchison simplex
         defaults to J.J.Egozcue orthonormal basis.
 
@@ -563,7 +560,8 @@ def ilr(mat, basis=None, check=True):
     """
     mat = closure(mat)
     if basis is None:
-        basis = clr_inv(_gram_schmidt_basis(mat.shape[-1]))
+        d = mat.shape[-1]
+        basis = _gram_schmidt_basis(d)  # dimension (d-1) x d
     else:
         if len(basis.shape) != 2:
             raise ValueError("Basis needs to be a 2D matrix, "
@@ -572,13 +570,12 @@ def ilr(mat, basis=None, check=True):
         if check:
             _check_orthogonality(basis)
 
-    return inner(mat, basis)
+    return clr(mat) @ basis.T
 
 
 @experimental(as_of="0.4.0")
 def ilr_inv(mat, basis=None, check=True):
-    r"""
-    Performs inverse isometric log ratio transform.
+    r""" Performs inverse isometric log ratio transform.
 
     This function transforms compositions from the real space to
     Aitchison geometry. The :math:`ilr^{-1}` transform is both an isometry,
@@ -628,8 +625,9 @@ def ilr_inv(mat, basis=None, check=True):
     the dimensions of the basis needs be `D-1 x D`, where rows represent
     basis vectors, and the columns represent proportions.
     """
-
+    mat = np.atleast_2d(mat)
     if basis is None:
+        # dimension d-1 x d basis
         basis = _gram_schmidt_basis(mat.shape[-1] + 1)
     else:
         if len(basis.shape) != 2:
@@ -640,15 +638,14 @@ def ilr_inv(mat, basis=None, check=True):
             _check_orthogonality(basis)
         # this is necessary, since the clr function
         # performs np.squeeze()
-        basis = np.atleast_2d(clr(basis))
+        basis = np.atleast_2d(basis)
 
-    return clr_inv(np.dot(mat, basis))
+    return clr_inv(mat @ basis)
 
 
 @experimental(as_of="0.5.5")
 def alr(mat, denominator_idx=0):
-    r"""
-    Performs additive log ratio transformation.
+    r""" Performs additive log ratio transformation.
 
     This function transforms compositions from a D-part Aitchison simplex to
     a non-isometric real space of D-1 dimensions. The argument
@@ -709,8 +706,7 @@ def alr(mat, denominator_idx=0):
 
 @experimental(as_of="0.5.5")
 def alr_inv(mat, denominator_idx=0):
-    r"""
-    Performs inverse additive log ratio transform.
+    r""" Performs inverse additive log ratio transform.
 
     This function transforms compositions from the non-isometric real space of
     alrs to Aitchison geometry.
@@ -807,6 +803,383 @@ def centralize(mat):
     return perturb_inv(mat, cen)
 
 
+@experimental(as_of="0.5.7")
+def _vlr(x: np.array, y: np.array, ddof: int):
+    r""" Calculates variance log ratio
+
+    Parameters
+    ----------
+    x : array_like, float
+       a 1-dimensional vector of proportions
+    y : array_like, float
+       a 1-dimensional vector of proportions
+    ddof: int
+        degrees of freedom
+
+    Returns
+    -------
+    float
+         variance log ratio value
+
+    References
+    ----------
+    .. [1] V. Lovell D, Pawlowsky-Glahn V, Egozcue JJ, Marguerat S,
+           Bähler J (2015) Proportionality: A Valid Alternative to
+           Correlation for Relative Data. PLoS Comput Biol 11(3): e1004075.
+           https://doi.org/10.1371/journal.pcbi.1004075
+    .. [2] Erb, I., Notredame, C.
+           How should we measure proportionality on relative gene
+           expression data?. Theory Biosci. 135, 21–36 (2016).
+           https://doi.org/10.1007/s12064-015-0220-8
+    """
+
+    # Log transformation
+    x = np.log(x)
+    y = np.log(y)
+
+    # Variance log ratio
+    return np.var(x - y, ddof=ddof)
+
+
+@experimental(as_of="0.5.7")
+def _robust_vlr(x: np.ndarray, y: np.ndarray, ddof: int):
+    r""" Calculates variance log ratio while masking zeros
+
+    Parameters
+    ----------
+    x : array_like, float
+       a 1-dimensional vector of proportions
+    y : array_like, float
+       a 1-dimensional vector of proportions
+    ddof: int
+        degrees of freedom
+
+    Returns
+    -------
+    float
+         variance log ratio value
+
+
+    References
+    ----------
+    .. [1] V. Lovell D, Pawlowsky-Glahn V, Egozcue JJ, Marguerat S,
+           Bähler J (2015) Proportionality: A Valid Alternative to
+           Correlation for Relative Data. PLoS Comput Biol 11(3): e1004075.
+           https://doi.org/10.1371/journal.pcbi.1004075
+    .. [2] Erb, I., Notredame, C.
+           How should we measure proportionality on relative gene
+           expression data?. Theory Biosci. 135, 21–36 (2016).
+           https://doi.org/10.1007/s12064-015-0220-8
+    """
+
+    # Mask zeros
+    x = np.ma.masked_array(x, mask=x == 0)
+    y = np.ma.masked_array(y, mask=y == 0)
+
+    # Log transformation
+    x = np.ma.log(x)
+    y = np.ma.log(y)
+
+    # Variance log ratio
+    return np.ma.var(x - y, ddof=ddof)
+
+
+@experimental(as_of="0.5.7")
+def vlr(x: np.ndarray, y: np.ndarray, ddof: int = 1, robust: bool = False):
+    r""" Calculates variance log ratio
+
+    Parameters
+    ----------
+    x : array_like, float
+       a 1-dimensional vector of proportions
+    y : array_like, float
+       a 1-dimensional vector of proportions
+
+    ddof: int
+        degrees of freedom
+
+    robust: bool
+        mask zeros at the cost of performance
+
+    Returns
+    -------
+    float
+         variance log ratio value
+
+    Examples
+    --------
+    No zeros
+    >>> import numpy as np
+    >>> from skbio.stats.composition import vlr
+    >>> x = np.exp([1,2,3])
+    >>> y = np.exp([2,3,4])
+    >>> vlr(x,y)
+    0.0
+
+    References
+    ----------
+    .. [1] V. Lovell D, Pawlowsky-Glahn V, Egozcue JJ, Marguerat S,
+           Bähler J (2015) Proportionality: A Valid Alternative to
+           Correlation for Relative Data. PLoS Comput Biol 11(3): e1004075.
+           https://doi.org/10.1371/journal.pcbi.1004075
+    .. [2] Erb, I., Notredame, C.
+           How should we measure proportionality on relative gene
+           expression data?. Theory Biosci. 135, 21–36 (2016).
+           https://doi.org/10.1007/s12064-015-0220-8
+    """
+    # Convert array_like to numpy array
+    x = closure(x)
+    y = closure(y)
+
+    # Set up input and parameters
+    kwargs = {
+        "x": x,
+        "y": y,
+        "ddof": ddof,
+    }
+
+    # Run backend function
+    if robust:
+        return _robust_vlr(**kwargs)
+    else:
+        return _vlr(**kwargs)
+
+
+@experimental(as_of="0.5.7")
+def _pairwise_vlr(mat: np.ndarray, ddof: int):
+    r""" Performs pairwise variance log ratio transformation
+
+    Parameters
+    ----------
+    mat : array_like, float
+       a matrix of proportions where
+       rows = compositions and
+       columns = components
+    ids : array_like, str
+        component names
+    ddof : int
+        degrees of freedom
+
+    Returns
+    -------
+    skbio.DistanceMatrix
+         distance matrix of variance log ratio values
+
+
+    References
+    ----------
+    .. [1] V. Lovell D, Pawlowsky-Glahn V, Egozcue JJ, Marguerat S,
+           Bähler J (2015) Proportionality: A Valid Alternative to
+           Correlation for Relative Data. PLoS Comput Biol 11(3): e1004075.
+           https://doi.org/10.1371/journal.pcbi.1004075
+    .. [2] Erb, I., Notredame, C.
+           How should we measure proportionality on relative gene
+           expression data?. Theory Biosci. 135, 21–36 (2016).
+           https://doi.org/10.1007/s12064-015-0220-8
+    """
+
+    # Log Transform
+    X_log = np.log(mat)
+
+    # Variance Log Ratio
+    covariance = np.cov(X_log.T, ddof=ddof)
+    diagonal = np.diagonal(covariance)
+    vlr_data = -2*covariance + diagonal[:, np.newaxis] + diagonal
+    return vlr_data
+
+
+@experimental(as_of="0.5.7")
+def pairwise_vlr(mat,
+                 ids=None,
+                 ddof: int = 1,
+                 robust: bool = False,
+                 validate: bool = True):
+
+    r""" Performs pairwise variance log ratio transformation
+
+    Parameters
+    ----------
+    mat: array_like, float
+       a matrix of proportions where
+       rows = compositions and
+       columns = components
+    ids: array_like, str
+        Component names
+    ddof: int
+        Degrees of freedom
+    robust: bool
+        Mask zeros at the cost of performance
+    validate: bool
+        Whether to validate the distance matrix after construction.
+
+    Returns
+    -------
+    skbio.DistanceMatrix if validate=True
+         distance matrix of variance log ratio values
+    skbio.DissimilarityMatrix if validate=False
+         dissimilarity matrix of variance log ratio values
+
+    Examples
+    --------
+    import numpy as np
+    from skbio.stats.composition import pairwise_vlr
+    >>> mat = np.array([np.exp([1,2,2]),
+    ...                 np.exp([2,3,6]),
+    ...                 np.exp([2,3,12])]).T
+    >>> dism = pairwise_vlr(mat)
+    >>> dism.redundant_form()
+    array([[  0.,   3.,  27.],
+           [  3.,   0.,  12.],
+           [ 27.,  12.,   0.]])
+
+
+    References
+    ----------
+    .. [1] V. Lovell D, Pawlowsky-Glahn V, Egozcue JJ, Marguerat S,
+           Bähler J (2015) Proportionality: A Valid Alternative to
+           Correlation for Relative Data. PLoS Comput Biol 11(3): e1004075.
+           https://doi.org/10.1371/journal.pcbi.1004075
+    .. [2] Erb, I., Notredame, C.
+           How should we measure proportionality on relative gene
+           expression data?. Theory Biosci. 135, 21–36 (2016).
+           https://doi.org/10.1007/s12064-015-0220-8
+    """
+
+    # Mask zeros
+    mat = closure(mat.astype(np.float64))
+
+    # Set up input and parameters
+    kwargs = {
+        "mat": mat,
+        "ddof": ddof,
+    }
+
+    # Variance Log Ratio
+    if robust:
+        raise NotImplementedError(
+            'Pairwise version of robust VLR not implemented.')
+    else:
+        vlr_data = _pairwise_vlr(**kwargs)
+
+    # Return distance matrix
+    if validate:
+        vlr_data = 0.5 * (vlr_data + vlr_data.T)
+        return DistanceMatrix(vlr_data, ids=ids)
+
+    # Return dissimilarity matrix
+    else:
+        return DistanceMatrix(vlr_data, ids=ids, validate=False)
+
+
+@experimental(as_of="0.5.8")
+def tree_basis(tree):
+    r""" Calculates sparse representation of an ilr basis from a tree.
+
+    This computes an orthonormal basis specified from a bifurcating tree.
+
+    Parameters
+    ----------
+    tree : skbio.TreeNode
+        Input bifurcating tree.  Must be strictly bifurcating
+        (i.e. every internal node needs to have exactly 2 children).
+        This is used to specify the ilr basis.
+
+    Returns
+    -------
+    scipy.sparse.coo_matrix
+       The ilr basis required to perform the ilr_inv transform.
+       This is also known as the sequential binary partition.
+       Note that this matrix is represented in clr coordinates.
+    nodes : list, str
+        List of tree nodes indicating the ordering in the basis.
+
+    Raises
+    ------
+    ValueError
+        The tree doesn't contain two branches.
+
+    >>> from skbio import TreeNode
+    >>> tree = u"((b,c)a, d)root;"
+    >>> t = TreeNode.read([tree])
+    >>> basis, nodes = tree_basis(t)
+    >>> basis.toarray()
+    array([[-0.40824829, -0.40824829,  0.81649658],
+           [-0.70710678,  0.70710678,  0.        ]])
+
+    """
+    # Specifies which child is numerator and denominator
+    # within any given node in a tree.
+    NUMERATOR = 1
+    DENOMINATOR = 0
+    # this is inspired by @wasade in
+    # https://github.com/biocore/gneiss/pull/8
+    t = tree.copy()
+    D = len(list(tree.tips()))
+    # calculate number of tips under each node
+    for n in t.postorder(include_self=True):
+        if n.is_tip():
+            n._tip_count = 1
+        else:
+            if len(n.children) == 2:
+                left, right = n.children[NUMERATOR], n.children[DENOMINATOR],
+            else:
+                raise ValueError("Not a strictly bifurcating tree.")
+            n._tip_count = left._tip_count + right._tip_count
+
+    # calculate k, r, s, t coordinate for each node
+    left, right = t.children[NUMERATOR], t.children[DENOMINATOR],
+    t._k, t._r, t._s, t._t = 0, left._tip_count, right._tip_count, 0
+    for n in t.preorder(include_self=False):
+        if n.is_tip():
+            n._k, n._r, n._s, n._t = 0, 0, 0, 0
+
+        elif n == n.parent.children[NUMERATOR]:
+            n._k = n.parent._k
+            n._r = n.children[NUMERATOR]._tip_count
+            n._s = n.children[DENOMINATOR]._tip_count
+            n._t = n.parent._s + n.parent._t
+        elif n == n.parent.children[DENOMINATOR]:
+            n._k = n.parent._r + n.parent._k
+            n._r = n.children[NUMERATOR]._tip_count
+            n._s = n.children[DENOMINATOR]._tip_count
+            n._t = n.parent._t
+        else:
+            raise ValueError("Tree topology is not correct.")
+
+    # navigate through tree to build the basis in a sparse matrix form
+    value = []
+    row, col = [], []
+    nodes = []
+    i = 0
+
+    for n in t.levelorder(include_self=True):
+
+        if n.is_tip():
+            continue
+
+        for j in range(n._k, n._k + n._r):
+            row.append(i)
+            # consider tips in reverse order. May want to rethink
+            # this orientation in the future.
+            col.append(D - 1 - j)
+            A = np.sqrt(n._s / (n._r * (n._s + n._r)))
+
+            value.append(A)
+
+        for j in range(n._k + n._r, n._k + n._r + n._s):
+            row.append(i)
+            col.append(D - 1 - j)
+            B = -np.sqrt(n._r / (n._s * (n._s + n._r)))
+
+            value.append(B)
+        i += 1
+        nodes.append(n.name)
+
+    basis = coo_matrix((value, (row, col)), shape=(D - 1, D))
+
+    return basis, nodes
+
+
 @experimental(as_of="0.4.1")
 def ancom(table, grouping,
           alpha=0.05,
@@ -1233,6 +1606,11 @@ def _gram_schmidt_basis(n):
     ----------
     n : int
         Dimension of the Aitchison simplex
+
+    Returns
+    -------
+    basis : np.array
+        Dimension (n-1) x n basis matrix
     """
     basis = np.zeros((n, n-1))
     for j in range(n-1):
@@ -1288,10 +1666,10 @@ def sbp_basis(sbp):
     ...                 [0, 0, 0, 1,-1]])
     ...
     >>> sbp_basis(sbp)
-    array([[ 0.31209907,  0.31209907,  0.12526729,  0.12526729,  0.12526729],
-           [ 0.36733337,  0.08930489,  0.18112058,  0.18112058,  0.18112058],
-           [ 0.17882092,  0.17882092,  0.40459293,  0.11888261,  0.11888261],
-           [ 0.18112058,  0.18112058,  0.18112058,  0.36733337,  0.08930489]])
+    array([[ 0.54772256,  0.54772256, -0.36514837, -0.36514837, -0.36514837],
+           [ 0.70710678, -0.70710678,  0.        ,  0.        ,  0.        ],
+           [ 0.        ,  0.        ,  0.81649658, -0.40824829, -0.40824829],
+           [ 0.        ,  0.        ,  0.        ,  0.70710678, -0.70710678]])
 
     References
     ----------
@@ -1311,7 +1689,7 @@ def sbp_basis(sbp):
     for i in range(0, sbp.shape[0]):
         psi[i, :] = sbp[i, :] * np.sqrt((n_neg[i] / n_pos[i])**sbp[i, :] /
                                         np.sum(np.abs(sbp[i, :])))
-    return clr_inv(psi)
+    return psi
 
 
 def _check_orthogonality(basis):
@@ -1322,9 +1700,9 @@ def _check_orthogonality(basis):
     Parameters
     ----------
     basis: numpy.ndarray
-        basis in the Aitchison simplex
+        basis in the Aitchison simplex of dimension d-1 x d
     """
     basis = np.atleast_2d(basis)
-    if not np.allclose(inner(basis, basis), np.identity(len(basis)),
+    if not np.allclose(basis @ basis.T, np.identity(len(basis)),
                        rtol=1e-4, atol=1e-6):
-        raise ValueError("Aitchison basis is not orthonormal")
+        raise ValueError("Basis is not orthonormal")
