@@ -3,15 +3,17 @@
 #
 # Distributed under the terms of the Modified BSD License.
 #
-# The full license is in the file COPYING.txt, distributed with this software.
+# The full license is in the file LICENSE.txt, distributed with this software.
 # ----------------------------------------------------------------------------
 
 from functools import partial
 
 import numpy as np
 
-from ._base import (_preprocess_input, _run_monte_carlo_stats, _build_results)
+from ._base import (_preprocess_input_sng, _run_monte_carlo_stats,
+                    _build_results, DistanceMatrix)
 from skbio.util._decorator import experimental
+from ._cutils import permanova_f_stat_sW_cy
 
 
 @experimental(as_of="0.4.0")
@@ -89,15 +91,22 @@ def permanova(distance_matrix, grouping, column=None, permutations=999):
     provide similar interfaces).
 
     """
-    sample_size, num_groups, grouping, tri_idxs, distances = _preprocess_input(
-        distance_matrix, grouping, column)
+    if not isinstance(distance_matrix, DistanceMatrix):
+        raise TypeError("Input must be a DistanceMatrix.")
+    sample_size = distance_matrix.shape[0]
+
+    num_groups, grouping = _preprocess_input_sng(
+        distance_matrix.ids, sample_size, grouping, column)
 
     # Calculate number of objects in each group.
     group_sizes = np.bincount(grouping)
-    s_T = (distances ** 2).sum() / sample_size
+    s_T = (distance_matrix[:] ** 2).sum() / sample_size
+    # we are going over the whole matrix, instead of just upper triangle
+    # so cut in half
+    s_T /= 2.0
 
     test_stat_function = partial(_compute_f_stat, sample_size, num_groups,
-                                 tri_idxs, distances, group_sizes, s_T)
+                                 distance_matrix, group_sizes, s_T)
     stat, p_value = _run_monte_carlo_stats(test_stat_function, grouping,
                                            permutations)
 
@@ -105,31 +114,13 @@ def permanova(distance_matrix, grouping, column=None, permutations=999):
                           stat, p_value, permutations)
 
 
-def _compute_f_stat(sample_size, num_groups, tri_idxs, distances, group_sizes,
+def _compute_f_stat(sample_size, num_groups, distance_matrix, group_sizes,
                     s_T, grouping):
     """Compute PERMANOVA pseudo-F statistic."""
-    # Create a matrix where objects in the same group are marked with the group
-    # index (e.g. 0, 1, 2, etc.). objects that are not in the same group are
-    # marked with -1.
-    grouping_matrix = -1 * np.ones((sample_size, sample_size), dtype=int)
-    for group_idx in range(num_groups):
-        within_indices = _index_combinations(
-            np.where(grouping == group_idx)[0])
-        grouping_matrix[within_indices] = group_idx
-
-    # Extract upper triangle (in same order as distances were extracted
-    # from full distance matrix).
-    grouping_tri = grouping_matrix[tri_idxs]
 
     # Calculate s_W for each group, accounting for different group sizes.
-    s_W = 0
-    for i in range(num_groups):
-        s_W += (distances[grouping_tri == i] ** 2).sum() / group_sizes[i]
+    s_W = permanova_f_stat_sW_cy(distance_matrix.data,
+                                 group_sizes, grouping)
 
     s_A = s_T - s_W
     return (s_A / (num_groups - 1)) / (s_W / (sample_size - num_groups))
-
-
-def _index_combinations(indices):
-    # Modified from http://stackoverflow.com/a/11144716
-    return np.tile(indices, len(indices)), np.repeat(indices, len(indices))
