@@ -3,7 +3,7 @@
 #
 # Distributed under the terms of the Modified BSD License.
 #
-# The full license is in the file COPYING.txt, distributed with this software.
+# The full license is in the file LICENSE.txt, distributed with this software.
 # ----------------------------------------------------------------------------
 
 import io
@@ -24,11 +24,12 @@ from skbio.stats.distance import (
     DissimilarityMatrix, randdm)
 from skbio.stats.distance._base import (_preprocess_input,
                                         _run_monte_carlo_stats)
+from skbio.stats.distance._utils import is_symmetric_and_hollow
 from skbio.util import assert_data_frame_almost_equal
 from skbio.util._testing import assert_series_almost_equal
 
 
-class DissimilarityMatrixTestData(TestCase):
+class DissimilarityMatrixTestData:
     def setUp(self):
         self.dm_1x1_data = [[0.0]]
         self.dm_2x2_data = [[0.0, 0.123], [0.123, 0.0]]
@@ -42,28 +43,44 @@ class DissimilarityMatrixTestData(TestCase):
                             [8, 9, 1, 2, 0]]
 
 
-class DissimilarityMatrixTests(DissimilarityMatrixTestData):
-    def setUp(self):
-        super(DissimilarityMatrixTests, self).setUp()
+class DissimilarityMatrixTestBase(DissimilarityMatrixTestData):
+    matobj = None
 
-        self.dm_1x1 = DissimilarityMatrix(self.dm_1x1_data, ['a'])
-        self.dm_2x2 = DissimilarityMatrix(self.dm_2x2_data, ['a', 'b'])
-        self.dm_2x2_asym = DissimilarityMatrix(self.dm_2x2_asym_data,
-                                               ['a', 'b'])
-        self.dm_3x3 = DissimilarityMatrix(self.dm_3x3_data, ['a', 'b', 'c'])
-        self.dm_5x5 = DissimilarityMatrix(self.dm_5x5_data, list('abcde'))
+    def setUp(self):
+        super(DissimilarityMatrixTestBase, self).setUp()
+        self.dm_1x1 = self.matobj(self.dm_1x1_data, ['a'])
+        self.dm_2x2 = self.matobj(self.dm_2x2_data, ['a', 'b'])
+        self.dm_2x2_asym = self.matobj(self.dm_2x2_asym_data,
+                                       ['a', 'b'])
+        self.dm_3x3 = self.matobj(self.dm_3x3_data, ['a', 'b', 'c'])
+        self.dm_5x5 = self.matobj(self.dm_5x5_data, list('abcde'))
 
         self.dms = [self.dm_1x1, self.dm_2x2, self.dm_2x2_asym, self.dm_3x3]
         self.dm_shapes = [(1, 1), (2, 2), (2, 2), (3, 3)]
         self.dm_sizes = [1, 4, 4, 9]
         self.dm_transposes = [
             self.dm_1x1, self.dm_2x2,
-            DissimilarityMatrix([[0, -2], [1, 0]], ['a', 'b']), self.dm_3x3]
+            self.matobj([[0, -2], [1, 0]], ['a', 'b']), self.dm_3x3]
         self.dm_redundant_forms = [np.array(self.dm_1x1_data),
                                    np.array(self.dm_2x2_data),
                                    np.array(self.dm_2x2_asym_data),
                                    np.array(self.dm_3x3_data)]
 
+    def test_avoid_copy_on_construction(self):
+        # ((data, expect_copy))
+        tests = (([[0, 1], [1, 0]], True),
+                 ([(0, 1), (1, 0)], True),
+                 (((0, 1), (1, 0)), True),
+                 (np.array([[0, 1], [1, 0]], dtype='int'), True),
+                 (np.array([[0, 1], [1, 0]], dtype='float'), False),
+                 (np.array([[0, 1], [1, 0]], dtype=np.float32), False),
+                 (np.array([[0, 1], [1, 0]], dtype=np.float64), False),
+                 (np.array([[0, 1], [1, 0]], dtype='double'), False))
+
+        for data, expect in tests:
+            obj = DissimilarityMatrix(data)
+            self.assertEqual(id(obj.data) != id(data), expect)
+
     def test_within(self):
         exp = pd.DataFrame([['a', 'a', 0.0],
                             ['a', 'c', 4.2],
@@ -191,8 +208,8 @@ class DissimilarityMatrixTests(DissimilarityMatrixTestData):
         ids = ['foo', 'bar', 'baz']
 
         # DissimilarityMatrix -> DissimilarityMatrix
-        exp = DissimilarityMatrix(self.dm_3x3_data, ids)
-        obs = DissimilarityMatrix(self.dm_3x3, ids)
+        exp = self.matobj(self.dm_3x3_data, ids)
+        obs = self.matobj(self.dm_3x3, ids)
         self.assertEqual(obs, exp)
         # Test that copy of data is not made.
         self.assertTrue(obs.data is self.dm_3x3.data)
@@ -200,9 +217,9 @@ class DissimilarityMatrixTests(DissimilarityMatrixTestData):
         self.assertTrue(np.array_equal(obs.data, self.dm_3x3.data))
 
         # DistanceMatrix -> DissimilarityMatrix
-        exp = DissimilarityMatrix(self.dm_3x3_data, ids)
-        obs = DissimilarityMatrix(
-            DistanceMatrix(self.dm_3x3_data, ('a', 'b', 'c')), ids)
+        exp = self.matobj(self.dm_3x3_data, ids)
+        obs = self.matobj(
+            self.matobj(self.dm_3x3_data, ('a', 'b', 'c')), ids)
         self.assertEqual(obs, exp)
 
         # DissimilarityMatrix -> DistanceMatrix
@@ -211,116 +228,118 @@ class DissimilarityMatrixTests(DissimilarityMatrixTestData):
 
     def test_init_non_hollow_dm(self):
         data = [[1, 1], [1, 1]]
-        obs = DissimilarityMatrix(data, ['a', 'b'])
+        obs = self.matobj(data, ['a', 'b'])
         self.assertTrue(np.array_equal(obs.data, data))
+        data_hollow = skbio.stats.distance._utils.is_hollow(obs.data)
+        self.assertEqual(data_hollow, False)
 
     def test_init_no_ids(self):
-        exp = DissimilarityMatrix(self.dm_3x3_data, ('0', '1', '2'))
-        obs = DissimilarityMatrix(self.dm_3x3_data)
+        exp = self.matobj(self.dm_3x3_data, ('0', '1', '2'))
+        obs = self.matobj(self.dm_3x3_data)
         self.assertEqual(obs, exp)
         self.assertEqual(obs['1', '2'], 12.0)
 
     def test_init_invalid_input(self):
         # Empty data.
         with self.assertRaises(DissimilarityMatrixError):
-            DissimilarityMatrix([], [])
+            self.matobj([], [])
 
         # Another type of empty data.
         with self.assertRaises(DissimilarityMatrixError):
-            DissimilarityMatrix(np.empty((0, 0)), [])
+            self.matobj(np.empty((0, 0)), [])
 
         # Invalid number of dimensions.
         with self.assertRaises(DissimilarityMatrixError):
-            DissimilarityMatrix([1, 2, 3], ['a'])
+            self.matobj([1, 2, 3], ['a'])
 
         # Dimensions don't match.
         with self.assertRaises(DissimilarityMatrixError):
-            DissimilarityMatrix([[1, 2, 3]], ['a'])
+            self.matobj([[1, 2, 3]], ['a'])
 
         data = [[0, 1], [1, 0]]
 
         # Duplicate IDs.
         with self.assertRaises(DissimilarityMatrixError):
-            DissimilarityMatrix(data, ['a', 'a'])
+            self.matobj(data, ['a', 'a'])
 
         # Number of IDs don't match dimensions.
         with self.assertRaises(DissimilarityMatrixError):
-            DissimilarityMatrix(data, ['a', 'b', 'c'])
+            self.matobj(data, ['a', 'b', 'c'])
         with self.assertRaises(DissimilarityMatrixError):
-            DissimilarityMatrix(data, [])
+            self.matobj(data, [])
 
     def test_from_iterable_non_hollow_data(self):
         iterable = (x for x in range(4))
 
-        exp = DissimilarityMatrix([[1, 1, 1, 1],
-                                   [1, 1, 1, 1],
-                                   [1, 1, 1, 1],
-                                   [1, 1, 1, 1]])
-        res = DissimilarityMatrix.from_iterable(iterable, lambda a, b: 1)
+        exp = self.matobj([[1, 1, 1, 1],
+                           [1, 1, 1, 1],
+                           [1, 1, 1, 1],
+                           [1, 1, 1, 1]])
+        res = self.matobj.from_iterable(iterable, lambda a, b: 1)
         self.assertEqual(res, exp)
 
     def test_from_iterable_asymmetric_data(self):
         iterable = (x for x in range(4))
 
-        exp = DissimilarityMatrix([[0, 1, 2, 3],
-                                   [-1, 0, 1, 2],
-                                   [-2, -1, 0, 1],
-                                   [-3, -2, -1, 0]])
-        res = DissimilarityMatrix.from_iterable(iterable, lambda a, b: b - a)
+        exp = self.matobj([[0, 1, 2, 3],
+                           [-1, 0, 1, 2],
+                           [-2, -1, 0, 1],
+                           [-3, -2, -1, 0]])
+        res = self.matobj.from_iterable(iterable, lambda a, b: b - a)
         self.assertEqual(res, exp)
 
     def test_from_iterable_no_key(self):
         iterable = (x for x in range(4))
 
-        exp = DissimilarityMatrix([[0, 1, 2, 3],
-                                   [1, 0, 1, 2],
-                                   [2, 1, 0, 1],
-                                   [3, 2, 1, 0]])
-        res = DissimilarityMatrix.from_iterable(iterable,
-                                                lambda a, b: abs(b - a))
+        exp = self.matobj([[0, 1, 2, 3],
+                           [1, 0, 1, 2],
+                           [2, 1, 0, 1],
+                           [3, 2, 1, 0]])
+        res = self.matobj.from_iterable(iterable,
+                                        lambda a, b: abs(b - a))
         self.assertEqual(res, exp)
 
     def test_from_iterable_with_key(self):
         iterable = (x for x in range(4))
 
-        exp = DissimilarityMatrix([[0, 1, 2, 3],
-                                   [1, 0, 1, 2],
-                                   [2, 1, 0, 1],
-                                   [3, 2, 1, 0]], ['0', '1', '4', '9'])
-        res = DissimilarityMatrix.from_iterable(iterable,
-                                                lambda a, b: abs(b - a),
-                                                key=lambda x: str(x ** 2))
+        exp = self.matobj([[0, 1, 2, 3],
+                           [1, 0, 1, 2],
+                           [2, 1, 0, 1],
+                           [3, 2, 1, 0]], ['0', '1', '4', '9'])
+        res = self.matobj.from_iterable(iterable,
+                                        lambda a, b: abs(b - a),
+                                        key=lambda x: str(x ** 2))
         self.assertEqual(res, exp)
 
     def test_from_iterable_empty(self):
         with self.assertRaises(DissimilarityMatrixError):
-            DissimilarityMatrix.from_iterable([], lambda x: x)
+            self.matobj.from_iterable([], lambda x: x)
 
     def test_from_iterable_single(self):
-        exp = DissimilarityMatrix([[100]])
-        res = DissimilarityMatrix.from_iterable(["boo"], lambda a, b: 100)
+        exp = self.matobj([[100]])
+        res = self.matobj.from_iterable(["boo"], lambda a, b: 100)
         self.assertEqual(res, exp)
 
     def test_from_iterable_with_keys(self):
         iterable = (x for x in range(4))
 
-        exp = DissimilarityMatrix([[0, 1, 2, 3],
-                                   [1, 0, 1, 2],
-                                   [2, 1, 0, 1],
-                                   [3, 2, 1, 0]], ['0', '1', '4', '9'])
-        res = DissimilarityMatrix.from_iterable(iterable,
-                                                lambda a, b: abs(b - a),
-                                                keys=iter(['0', '1', '4', '9'])
-                                                )
+        exp = self.matobj([[0, 1, 2, 3],
+                           [1, 0, 1, 2],
+                           [2, 1, 0, 1],
+                           [3, 2, 1, 0]], ['0', '1', '4', '9'])
+        res = self.matobj.from_iterable(iterable,
+                                        lambda a, b: abs(b - a),
+                                        keys=iter(['0', '1', '4', '9'])
+                                        )
         self.assertEqual(res, exp)
 
     def test_from_iterable_with_key_and_keys(self):
         iterable = (x for x in range(4))
         with self.assertRaises(ValueError):
-            DissimilarityMatrix.from_iterable(iterable,
-                                              lambda a, b: abs(b - a),
-                                              key=str,
-                                              keys=['1', '2', '3', '4'])
+            self.matobj.from_iterable(iterable,
+                                      lambda a, b: abs(b - a),
+                                      key=str,
+                                      keys=['1', '2', '3', '4'])
 
     def test_from_iterable_scipy_hamming_metric_with_metadata(self):
         # test for #1254
@@ -331,13 +350,13 @@ class DissimilarityMatrixTests(DissimilarityMatrixTestData):
             Sequence('AAAA', positional_metadata={'qual': range(4)})
         ]
 
-        exp = DissimilarityMatrix([
+        exp = self.matobj([
             [0, 0.25, 0.75, 0.75],
             [0.25, 0.0, 0.5, 0.5],
             [0.75, 0.5, 0.0, 0.0],
             [0.75, 0.5, 0.0, 0.0]], ['a', 'b', 'c', 'd'])
 
-        dm = DissimilarityMatrix.from_iterable(
+        dm = self.matobj.from_iterable(
             seqs,
             metric=scipy.spatial.distance.hamming,
             keys=['a', 'b', 'c', 'd'])
@@ -353,13 +372,13 @@ class DissimilarityMatrixTests(DissimilarityMatrixTestData):
             Sequence('AAAA', positional_metadata={'qual': range(4)})
         ]
 
-        exp = DissimilarityMatrix([
+        exp = self.matobj([
             [0, 0.25, 0.75, 0.75],
             [0.25, 0.0, 0.5, 0.5],
             [0.75, 0.5, 0.0, 0.0],
             [0.75, 0.5, 0.0, 0.0]], ['a', 'b', 'c', 'd'])
 
-        dm = DissimilarityMatrix.from_iterable(
+        dm = self.matobj.from_iterable(
             seqs,
             metric=skbio.sequence.distance.hamming,
             keys=['a', 'b', 'c', 'd'])
@@ -459,48 +478,48 @@ class DissimilarityMatrixTests(DissimilarityMatrixTestData):
     def test_filter_reorder(self):
         # Don't filter anything, but reorder the distance matrix.
         order = ['c', 'a', 'b']
-        exp = DissimilarityMatrix(
+        exp = self.matobj(
             [[0, 4.2, 12], [4.2, 0, 0.01], [12, 0.01, 0]], order)
         obs = self.dm_3x3.filter(order)
         self.assertEqual(obs, exp)
 
     def test_filter_single_id(self):
         ids = ['b']
-        exp = DissimilarityMatrix([[0]], ids)
+        exp = self.matobj([[0]], ids)
         obs = self.dm_2x2_asym.filter(ids)
         self.assertEqual(obs, exp)
 
     def test_filter_asymmetric(self):
         # 2x2
         ids = ['b', 'a']
-        exp = DissimilarityMatrix([[0, -2], [1, 0]], ids)
+        exp = self.matobj([[0, -2], [1, 0]], ids)
         obs = self.dm_2x2_asym.filter(ids)
         self.assertEqual(obs, exp)
 
         # 3x3
-        dm = DissimilarityMatrix([[0, 10, 53], [42, 0, 22.5], [53, 1, 0]],
-                                 ('bro', 'brah', 'breh'))
+        dm = self.matobj([[0, 10, 53], [42, 0, 22.5], [53, 1, 0]],
+                         ('bro', 'brah', 'breh'))
         ids = ['breh', 'brah']
-        exp = DissimilarityMatrix([[0, 1], [22.5, 0]], ids)
+        exp = self.matobj([[0, 1], [22.5, 0]], ids)
         obs = dm.filter(ids)
         self.assertEqual(obs, exp)
 
     def test_filter_subset(self):
         ids = ('c', 'a')
-        exp = DissimilarityMatrix([[0, 4.2], [4.2, 0]], ids)
+        exp = self.matobj([[0, 4.2], [4.2, 0]], ids)
         obs = self.dm_3x3.filter(ids)
         self.assertEqual(obs, exp)
 
         ids = ('b', 'a')
-        exp = DissimilarityMatrix([[0, 0.01], [0.01, 0]], ids)
+        exp = self.matobj([[0, 0.01], [0.01, 0]], ids)
         obs = self.dm_3x3.filter(ids)
         self.assertEqual(obs, exp)
 
         # 4x4
-        dm = DissimilarityMatrix([[0, 1, 55, 7], [1, 0, 16, 1],
-                                  [55, 16, 0, 23], [7, 1, 23, 0]])
+        dm = self.matobj([[0, 1, 55, 7], [1, 0, 16, 1],
+                         [55, 16, 0, 23], [7, 1, 23, 0]])
         ids = np.asarray(['3', '0', '1'])
-        exp = DissimilarityMatrix([[0, 7, 1], [7, 0, 1], [1, 1, 0]], ids)
+        exp = self.matobj([[0, 7, 1], [7, 0, 1], [1, 1, 0]], ids)
         obs = dm.filter(ids)
         self.assertEqual(obs, exp)
 
@@ -515,7 +534,7 @@ class DissimilarityMatrixTests(DissimilarityMatrixTestData):
     def test_filter_missing_ids_strict_false(self):
         # no exception should be raised
         ids = ('c', 'a')
-        exp = DissimilarityMatrix([[0, 4.2], [4.2, 0]], ids)
+        exp = self.matobj([[0, 4.2], [4.2, 0]], ids)
         obs = self.dm_3x3.filter(['c', 'a', 'not found'], strict=False)
         self.assertEqual(obs, exp)
 
@@ -543,7 +562,7 @@ class DissimilarityMatrixTests(DissimilarityMatrixTestData):
         ids = ['0', 'one', '2', 'three', '4.000']
         data = ([0, 1, 2, 3, 4], [1, 0, 1, 2, 3], [2, 1, 0, 1, 2],
                 [3, 2, 1, 0, 1], [4, 3, 2, 1, 0])
-        dm = DissimilarityMatrix(data, ids)
+        dm = self.matobj(data, ids)
         fig = dm.plot(cmap='Reds', title='Testplot')
         self.assertIsInstance(fig, mpl.figure.Figure)
         axes = fig.get_axes()
@@ -592,7 +611,7 @@ class DissimilarityMatrixTests(DissimilarityMatrixTestData):
         assert_data_frame_almost_equal(df, exp)
 
     def test_to_data_frame_default_ids(self):
-        df = DissimilarityMatrix(self.dm_2x2_data).to_data_frame()
+        df = self.matobj(self.dm_2x2_data).to_data_frame()
         exp = pd.DataFrame([[0.0, 0.123],
                             [0.123, 0.0]],
                            index=['0', '1'], columns=['0', '1'])
@@ -702,14 +721,44 @@ class DissimilarityMatrixTests(DissimilarityMatrixTestData):
         with self.assertRaises(DissimilarityMatrixError):
             self.dm_3x3._validate(np.array([[0, 42], [42, 0]]), ['a', 'b'])
 
+    def test_validate_invalid_shape(self):
+        # first check it actually likes good matrices
+        self.dm_3x3._validate_shape(np.array([[0., 42.], [42., 0.]]))
+        # it checks just the shape, not the content
+        self.dm_3x3._validate_shape(np.array([[1., 2.], [3., 4.]]))
+        # empty array
+        with self.assertRaises(DissimilarityMatrixError):
+            self.dm_3x3._validate_shape(np.array([]))
+        # invalid shape
+        with self.assertRaises(DissimilarityMatrixError):
+            self.dm_3x3._validate_shape(np.array([[0., 42.],
+                                                  [42., 0.],
+                                                  [22., 22.]]))
+        with self.assertRaises(DissimilarityMatrixError):
+            self.dm_3x3._validate_shape(np.array([[[0., 42.], [42., 0.]],
+                                                  [[0., 24.], [24., 0.]]]))
+
+    def test_validate_invalid_ids(self):
+        # repeated ids
+        with self.assertRaises(DissimilarityMatrixError):
+            self.dm_3x3._validate_ids(self.dm_3x3.data, ['a', 'a'])
+        # empty ids
+        with self.assertRaises(DissimilarityMatrixError):
+            self.dm_3x3._validate_ids(self.dm_3x3.data, [])
+        # invalid shape
+        with self.assertRaises(DissimilarityMatrixError):
+            self.dm_3x3._validate_ids(self.dm_3x3.data, ['a', 'b', 'c', 'd'])
+
+
+class DistanceMatrixTestBase(DissimilarityMatrixTestData):
+    matobj = None
 
-class DistanceMatrixTests(DissimilarityMatrixTestData):
     def setUp(self):
-        super(DistanceMatrixTests, self).setUp()
+        super(DistanceMatrixTestBase, self).setUp()
 
-        self.dm_1x1 = DistanceMatrix(self.dm_1x1_data, ['a'])
-        self.dm_2x2 = DistanceMatrix(self.dm_2x2_data, ['a', 'b'])
-        self.dm_3x3 = DistanceMatrix(self.dm_3x3_data, ['a', 'b', 'c'])
+        self.dm_1x1 = self.matobj(self.dm_1x1_data, ['a'])
+        self.dm_2x2 = self.matobj(self.dm_2x2_data, ['a', 'b'])
+        self.dm_3x3 = self.matobj(self.dm_3x3_data, ['a', 'b', 'c'])
 
         self.dms = [self.dm_1x1, self.dm_2x2, self.dm_3x3]
         self.dm_condensed_forms = [np.array([]), np.array([0.123]),
@@ -717,117 +766,117 @@ class DistanceMatrixTests(DissimilarityMatrixTestData):
 
     def test_init_from_condensed_form(self):
         data = [1, 2, 3]
-        exp = DistanceMatrix([[0, 1, 2],
-                              [1, 0, 3],
-                              [2, 3, 0]], ['0', '1', '2'])
-        res = DistanceMatrix(data)
+        exp = self.matobj([[0, 1, 2],
+                           [1, 0, 3],
+                           [2, 3, 0]], ['0', '1', '2'])
+        res = self.matobj(data)
         self.assertEqual(exp, res)
 
     def test_init_invalid_input(self):
         # Asymmetric.
         data = [[0.0, 2.0], [1.0, 0.0]]
         with self.assertRaises(DistanceMatrixError):
-            DistanceMatrix(data, ['a', 'b'])
+            self.matobj(data, ['a', 'b'])
 
         # Non-hollow
         data = [[1.0, 2.0], [2.0, 1.0]]
         with self.assertRaises(DistanceMatrixError):
-            DistanceMatrix(data, ['a', 'b'])
+            self.matobj(data, ['a', 'b'])
 
         # Ensure that the superclass validation is still being performed.
         with self.assertRaises(DissimilarityMatrixError):
-            DistanceMatrix([[1, 2, 3]], ['a'])
+            self.matobj([[1, 2, 3]], ['a'])
 
     def test_init_nans(self):
         with self.assertRaisesRegex(DistanceMatrixError, r'NaNs'):
-            DistanceMatrix([[0.0, np.nan], [np.nan, 0.0]], ['a', 'b'])
+            self.matobj([[0.0, np.nan], [np.nan, 0.0]], ['a', 'b'])
 
     def test_from_iterable_no_key(self):
         iterable = (x for x in range(4))
 
-        exp = DistanceMatrix([[0, 1, 2, 3],
-                              [1, 0, 1, 2],
-                              [2, 1, 0, 1],
-                              [3, 2, 1, 0]])
-        res = DistanceMatrix.from_iterable(iterable, lambda a, b: abs(b - a))
+        exp = self.matobj([[0, 1, 2, 3],
+                           [1, 0, 1, 2],
+                           [2, 1, 0, 1],
+                           [3, 2, 1, 0]])
+        res = self.matobj.from_iterable(iterable, lambda a, b: abs(b - a))
         self.assertEqual(res, exp)
 
     def test_from_iterable_validate_equal_valid_data(self):
-        validate_true = DistanceMatrix.from_iterable((x for x in range(4)),
-                                                     lambda a, b: abs(b - a),
-                                                     validate=True)
-        validate_false = DistanceMatrix.from_iterable((x for x in range(4)),
-                                                      lambda a, b: abs(b - a),
-                                                      validate=False)
+        validate_true = self.matobj.from_iterable((x for x in range(4)),
+                                                  lambda a, b: abs(b - a),
+                                                  validate=True)
+        validate_false = self.matobj.from_iterable((x for x in range(4)),
+                                                   lambda a, b: abs(b - a),
+                                                   validate=False)
         self.assertEqual(validate_true, validate_false)
 
     def test_from_iterable_validate_false(self):
         iterable = (x for x in range(4))
 
-        exp = DistanceMatrix([[0, 1, 2, 3],
-                              [1, 0, 1, 2],
-                              [2, 1, 0, 1],
-                              [3, 2, 1, 0]])
-        res = DistanceMatrix.from_iterable(iterable, lambda a, b: abs(b - a),
-                                           validate=False)
+        exp = self.matobj([[0, 1, 2, 3],
+                           [1, 0, 1, 2],
+                           [2, 1, 0, 1],
+                           [3, 2, 1, 0]])
+        res = self.matobj.from_iterable(iterable, lambda a, b: abs(b - a),
+                                        validate=False)
         self.assertEqual(res, exp)
 
     def test_from_iterable_validate_non_hollow(self):
         iterable = (x for x in range(4))
         with self.assertRaises(DistanceMatrixError):
-            DistanceMatrix.from_iterable(iterable, lambda a, b: 1)
+            self.matobj.from_iterable(iterable, lambda a, b: 1)
 
     def test_from_iterable_validate_false_non_symmetric(self):
-        exp = DistanceMatrix([[0, 1, 2, 3],
-                              [1, 0, 1, 2],
-                              [2, 1, 0, 1],
-                              [3, 2, 1, 0]])
-        res = DistanceMatrix.from_iterable((x for x in range(4)),
-                                           lambda a, b: a - b,
-                                           validate=False)
+        exp = self.matobj([[0, 1, 2, 3],
+                           [1, 0, 1, 2],
+                           [2, 1, 0, 1],
+                           [3, 2, 1, 0]])
+        res = self.matobj.from_iterable((x for x in range(4)),
+                                        lambda a, b: a - b,
+                                        validate=False)
         self.assertEqual(res, exp)
 
     def test_from_iterable_validate_asym(self):
         iterable = (x for x in range(4))
         with self.assertRaises(DistanceMatrixError):
-            DistanceMatrix.from_iterable(iterable, lambda a, b: b - a)
+            self.matobj.from_iterable(iterable, lambda a, b: b - a)
 
     def test_from_iterable_with_key(self):
         iterable = (x for x in range(4))
 
-        exp = DistanceMatrix([[0, 1, 2, 3],
-                              [1, 0, 1, 2],
-                              [2, 1, 0, 1],
-                              [3, 2, 1, 0]], ['0', '1', '4', '9'])
-        res = DistanceMatrix.from_iterable(iterable, lambda a, b: abs(b - a),
-                                           key=lambda x: str(x**2))
+        exp = self.matobj([[0, 1, 2, 3],
+                           [1, 0, 1, 2],
+                           [2, 1, 0, 1],
+                           [3, 2, 1, 0]], ['0', '1', '4', '9'])
+        res = self.matobj.from_iterable(iterable, lambda a, b: abs(b - a),
+                                        key=lambda x: str(x**2))
         self.assertEqual(res, exp)
 
     def test_from_iterable_empty(self):
         with self.assertRaises(DissimilarityMatrixError):
-            DistanceMatrix.from_iterable([], lambda x: x)
+            self.matobj.from_iterable([], lambda x: x)
 
     def test_from_iterable_single(self):
-        exp = DistanceMatrix([[0]])
-        res = DistanceMatrix.from_iterable(["boo"], lambda a, b: 0)
+        exp = self.matobj([[0]])
+        res = self.matobj.from_iterable(["boo"], lambda a, b: 0)
         self.assertEqual(res, exp)
 
     def test_from_iterable_with_keys(self):
         iterable = (x for x in range(4))
 
-        exp = DistanceMatrix([[0, 1, 2, 3],
-                              [1, 0, 1, 2],
-                              [2, 1, 0, 1],
-                              [3, 2, 1, 0]], ['0', '1', '4', '9'])
-        res = DistanceMatrix.from_iterable(iterable, lambda a, b: abs(b - a),
-                                           keys=iter(['0', '1', '4', '9']))
+        exp = self.matobj([[0, 1, 2, 3],
+                           [1, 0, 1, 2],
+                           [2, 1, 0, 1],
+                           [3, 2, 1, 0]], ['0', '1', '4', '9'])
+        res = self.matobj.from_iterable(iterable, lambda a, b: abs(b - a),
+                                        keys=iter(['0', '1', '4', '9']))
         self.assertEqual(res, exp)
 
     def test_from_iterable_with_key_and_keys(self):
         iterable = (x for x in range(4))
         with self.assertRaises(ValueError):
-            DistanceMatrix.from_iterable(iterable, lambda a, b: abs(b - a),
-                                         key=str, keys=['1', '2', '3', '4'])
+            self.matobj.from_iterable(iterable, lambda a, b: abs(b - a),
+                                      key=str, keys=['1', '2', '3', '4'])
 
     def test_from_iterable_scipy_hamming_metric_with_metadata(self):
         # test for #1254
@@ -838,13 +887,13 @@ class DistanceMatrixTests(DissimilarityMatrixTestData):
             Sequence('AAAA', positional_metadata={'qual': range(4)})
         ]
 
-        exp = DistanceMatrix([
+        exp = self.matobj([
             [0, 0.25, 0.75, 0.75],
             [0.25, 0.0, 0.5, 0.5],
             [0.75, 0.5, 0.0, 0.0],
             [0.75, 0.5, 0.0, 0.0]], ['a', 'b', 'c', 'd'])
 
-        dm = DistanceMatrix.from_iterable(
+        dm = self.matobj.from_iterable(
             seqs,
             metric=scipy.spatial.distance.hamming,
             keys=['a', 'b', 'c', 'd'])
@@ -860,13 +909,13 @@ class DistanceMatrixTests(DissimilarityMatrixTestData):
             Sequence('AAAA', positional_metadata={'qual': range(4)})
         ]
 
-        exp = DistanceMatrix([
+        exp = self.matobj([
             [0, 0.25, 0.75, 0.75],
             [0.25, 0.0, 0.5, 0.5],
             [0.75, 0.5, 0.0, 0.0],
             [0.75, 0.5, 0.0, 0.0]], ['a', 'b', 'c', 'd'])
 
-        dm = DistanceMatrix.from_iterable(
+        dm = self.matobj.from_iterable(
             seqs,
             metric=skbio.sequence.distance.hamming,
             keys=['a', 'b', 'c', 'd'])
@@ -913,15 +962,15 @@ class DistanceMatrixTests(DissimilarityMatrixTestData):
 
         np.random.seed(0)
 
-        exp = DistanceMatrix([[0, 12, 4.2],
-                              [12, 0, 0.01],
-                              [4.2, 0.01, 0]], self.dm_3x3.ids)
+        exp = self.matobj([[0, 12, 4.2],
+                           [12, 0, 0.01],
+                           [4.2, 0.01, 0]], self.dm_3x3.ids)
         obs = self.dm_3x3.permute()
         self.assertEqual(obs, exp)
 
-        exp = DistanceMatrix([[0, 4.2, 12],
-                              [4.2, 0, 0.01],
-                              [12, 0.01, 0]], self.dm_3x3.ids)
+        exp = self.matobj([[0, 4.2, 12],
+                           [4.2, 0, 0.01],
+                           [12, 0.01, 0]], self.dm_3x3.ids)
         obs = self.dm_3x3.permute()
         self.assertEqual(obs, exp)
 
@@ -935,7 +984,7 @@ class DistanceMatrixTests(DissimilarityMatrixTestData):
     def test_to_series_1x1(self):
         series = self.dm_1x1.to_series()
 
-        exp = pd.Series([], index=[])
+        exp = pd.Series([], index=[], dtype='float64')
         assert_series_almost_equal(series, exp)
 
     def test_to_series_2x2(self):
@@ -945,7 +994,7 @@ class DistanceMatrixTests(DissimilarityMatrixTestData):
         assert_series_almost_equal(series, exp)
 
     def test_to_series_4x4(self):
-        dm = DistanceMatrix([
+        dm = self.matobj([
             [0.0, 0.2, 0.3, 0.4],
             [0.2, 0.0, 0.5, 0.6],
             [0.3, 0.5, 0.0, 0.7],
@@ -959,11 +1008,60 @@ class DistanceMatrixTests(DissimilarityMatrixTestData):
         assert_series_almost_equal(series, exp)
 
     def test_to_series_default_ids(self):
-        series = DistanceMatrix(self.dm_2x2_data).to_series()
+        series = self.matobj(self.dm_2x2_data).to_series()
 
         exp = pd.Series([0.123], index=pd.Index([('0', '1')]))
         assert_series_almost_equal(series, exp)
 
+    def test_validate_asym_shape(self):
+        # first check it actually likes good matrices
+        data_good = np.array([[0., 42.], [42., 0.]])
+        data_sym, data_hollow = is_symmetric_and_hollow(data_good)
+        self.assertEqual(data_sym, True)
+        del data_sym
+        self.assertEqual(data_hollow, True)
+        del data_hollow
+        data_sym = skbio.stats.distance._utils.is_symmetric(data_good)
+        self.assertEqual(data_sym, True)
+        del data_sym
+        data_hollow = skbio.stats.distance._utils.is_hollow(data_good)
+        self.assertEqual(data_hollow, True)
+        del data_hollow
+        self.dm_3x3._validate_shape(data_good)
+        del data_good
+
+        # _validate_shap checks just the shape, not the content
+        bad_data = np.array([[1., 2.], [3., 4.]])
+        data_sym, data_hollow = is_symmetric_and_hollow(bad_data)
+        self.assertEqual(data_sym, False)
+        del data_sym
+        self.assertEqual(data_hollow, False)
+        del data_hollow
+        data_sym = skbio.stats.distance._utils.is_symmetric(bad_data)
+        self.assertEqual(data_sym, False)
+        del data_sym
+        data_hollow = skbio.stats.distance._utils.is_hollow(bad_data)
+        self.assertEqual(data_hollow, False)
+        del data_hollow
+        self.dm_3x3._validate_shape(bad_data)
+        del bad_data
+
+        # re-try with partially bad data
+        bad_data = np.array([[0., 2.], [3., 0.]])
+        data_sym, data_hollow = is_symmetric_and_hollow(bad_data)
+        self.assertEqual(data_sym, False)
+        del data_sym
+        self.assertEqual(data_hollow, True)
+        del data_hollow
+        data_sym = skbio.stats.distance._utils.is_symmetric(bad_data)
+        self.assertEqual(data_sym, False)
+        del data_sym
+        data_hollow = skbio.stats.distance._utils.is_hollow(bad_data)
+        self.assertEqual(data_hollow, True)
+        del data_hollow
+        self.dm_3x3._validate_shape(bad_data)
+        del bad_data
+
 
 class RandomDistanceMatrixTests(TestCase):
     def test_default_usage(self):
@@ -1102,5 +1200,19 @@ class CategoricalStatsHelperFunctionTests(TestCase):
             _run_monte_carlo_stats(lambda e: 42, self.grouping, -1)
 
 
+class DissimilarityMatrixTests(DissimilarityMatrixTestBase, TestCase):
+    matobj = DissimilarityMatrix
+
+    def setUp(self):
+        super(DissimilarityMatrixTests, self).setUp()
+
+
+class DistanceMatrixTests(DistanceMatrixTestBase, TestCase):
+    matobj = DistanceMatrix
+
+    def setUp(self):
+        super(DistanceMatrixTests, self).setUp()
+
+
 if __name__ == '__main__':
     main()
