@@ -15,12 +15,29 @@ from numpy.random import normal
 import pandas as pd
 import scipy
 import copy
+from skbio import TreeNode
 from skbio.util import assert_data_frame_almost_equal
+from skbio.stats.distance import DistanceMatrixError
 from skbio.stats.composition import (closure, multiplicative_replacement,
                                      perturb, perturb_inv, power, inner,
                                      clr, clr_inv, ilr, ilr_inv, alr, alr_inv,
                                      sbp_basis, _gram_schmidt_basis,
-                                     centralize, _holm_bonferroni, ancom)
+                                     centralize, _holm_bonferroni, ancom,
+                                     vlr, pairwise_vlr, tree_basis)
+
+from scipy.sparse import coo_matrix
+
+
+def assert_coo_allclose(res, exp, rtol=1e-7, atol=1e-7):
+    res_data = np.vstack((res.row, res.col, res.data)).T
+    exp_data = np.vstack((exp.row, exp.col, exp.data)).T
+
+    # sort by row and col
+    res_data = res_data[res_data[:, 1].argsort()]
+    res_data = res_data[res_data[:, 0].argsort()]
+    exp_data = exp_data[exp_data[:, 1].argsort()]
+    exp_data = exp_data[exp_data[:, 0].argsort()]
+    npt.assert_allclose(res_data, exp_data, rtol=rtol, atol=atol)
 
 
 class CompositionTests(TestCase):
@@ -313,13 +330,13 @@ class CompositionTests(TestCase):
                           [1.28282828, 9.81818182],
                           [1.42424242, 9.72727273],
                           [1.56565657, 9.63636364]])
-        basis = np.array([[0.80442968, 0.19557032]])
+        basis = np.atleast_2d(clr([[0.80442968, 0.19557032]]))
         res = ilr(table, basis=basis)
-        exp = np.array([np.log(1/10)*np.sqrt(1/2),
-                        np.log(1.14141414 / 9.90909091)*np.sqrt(1/2),
-                        np.log(1.28282828 / 9.81818182)*np.sqrt(1/2),
-                        np.log(1.42424242 / 9.72727273)*np.sqrt(1/2),
-                        np.log(1.56565657 / 9.63636364)*np.sqrt(1/2)])
+        exp = np.array([[np.log(1/10)*np.sqrt(1/2)],
+                        [np.log(1.14141414 / 9.90909091)*np.sqrt(1/2)],
+                        [np.log(1.28282828 / 9.81818182)*np.sqrt(1/2)],
+                        [np.log(1.42424242 / 9.72727273)*np.sqrt(1/2)],
+                        [np.log(1.56565657 / 9.63636364)*np.sqrt(1/2)]])
 
         npt.assert_allclose(res, exp)
 
@@ -352,14 +369,15 @@ class CompositionTests(TestCase):
     def test_ilr_basis_isomorphism(self):
         # tests to make sure that the isomorphism holds
         # with the introduction of the basis.
-        basis = np.array([[0.80442968, 0.19557032]])
+        basis = np.atleast_2d(clr([[0.80442968, 0.19557032]]))
         table = np.array([[np.log(1/10)*np.sqrt(1/2),
                            np.log(1.14141414 / 9.90909091)*np.sqrt(1/2),
                            np.log(1.28282828 / 9.81818182)*np.sqrt(1/2),
                            np.log(1.42424242 / 9.72727273)*np.sqrt(1/2),
                            np.log(1.56565657 / 9.63636364)*np.sqrt(1/2)]]).T
-        res = ilr(ilr_inv(table, basis=basis), basis=basis)
-        npt.assert_allclose(res, table.squeeze())
+        lr = ilr_inv(table, basis=basis)
+        res = ilr(lr, basis=basis)
+        npt.assert_allclose(res, table)
 
         table = np.array([[1., 10.],
                           [1.14141414, 9.90909091],
@@ -367,7 +385,7 @@ class CompositionTests(TestCase):
                           [1.42424242, 9.72727273],
                           [1.56565657, 9.63636364]])
 
-        res = ilr_inv(np.atleast_2d(ilr(table, basis=basis)).T, basis=basis)
+        res = ilr_inv(ilr(table, basis=basis), basis=basis)
         npt.assert_allclose(res, closure(table.squeeze()))
 
     def test_ilr_inv_basis(self):
@@ -376,17 +394,18 @@ class CompositionTests(TestCase):
                                 [1.28282828, 9.81818182],
                                 [1.42424242, 9.72727273],
                                 [1.56565657, 9.63636364]]))
-        basis = np.array([[0.80442968, 0.19557032]])
+        basis = np.atleast_2d(clr([[0.80442968, 0.19557032]]))
         table = np.array([[np.log(1/10)*np.sqrt(1/2),
                            np.log(1.14141414 / 9.90909091)*np.sqrt(1/2),
                            np.log(1.28282828 / 9.81818182)*np.sqrt(1/2),
                            np.log(1.42424242 / 9.72727273)*np.sqrt(1/2),
                            np.log(1.56565657 / 9.63636364)*np.sqrt(1/2)]]).T
+
         res = ilr_inv(table, basis=basis)
         npt.assert_allclose(res, exp)
 
     def test_ilr_inv_basis_one_dimension_error(self):
-        basis = clr(np.array([[0.80442968, 0.19557032]]))
+        basis = clr([0.80442968, 0.19557032])
         table = np.array([[np.log(1/10)*np.sqrt(1/2),
                            np.log(1.14141414 / 9.90909091)*np.sqrt(1/2),
                            np.log(1.28282828 / 9.81818182)*np.sqrt(1/2),
@@ -457,7 +476,7 @@ class CompositionTests(TestCase):
             alr_inv(self.bad2)
 
     def test_sbp_basis_gram_schmidt(self):
-        gsbasis = clr_inv(_gram_schmidt_basis(5))
+        gsbasis = _gram_schmidt_basis(5)
         sbp = np.array([[1, -1, 0, 0, 0],
                         [1, 1, -1, 0, 0],
                         [1, 1, 1, -1, 0],
@@ -490,8 +509,59 @@ class CompositionTests(TestCase):
                     psi[i, j] = np.sqrt(s[i]/(r[i]*(r[i]+s[i])))
                 elif sbp[i, j] == -1:
                     psi[i, j] = -np.sqrt(r[i]/(s[i]*(r[i]+s[i])))
-        basis_byhand = clr_inv(psi)
-        npt.assert_allclose(basis_byhand, sbpbasis)
+        npt.assert_allclose(psi, sbpbasis)
+
+
+class TestTreeBasis(TestCase):
+
+    def test_tree_basis_base_case(self):
+        tree = u"(a,b);"
+        t = TreeNode.read([tree])
+
+        exp_basis = coo_matrix(
+            np.array([[-np.sqrt(1. / 2),
+                       np.sqrt(1. / 2)]]))
+        exp_keys = [t.name]
+        res_basis, res_keys = tree_basis(t)
+
+        assert_coo_allclose(exp_basis, res_basis)
+        self.assertListEqual(exp_keys, res_keys)
+
+    def test_tree_basis_invalid(self):
+        with self.assertRaises(ValueError):
+            tree = u"(a,b,c);"
+            t = TreeNode.read([tree])
+            tree_basis(t)
+
+    def test_tree_basis_unbalanced(self):
+        tree = u"((a,b)c, d);"
+        t = TreeNode.read([tree])
+        exp_basis = coo_matrix(np.array(
+            [[-np.sqrt(1. / 6), -np.sqrt(1. / 6), np.sqrt(2. / 3)],
+             [-np.sqrt(1. / 2), np.sqrt(1. / 2), 0]]
+        ))
+        exp_keys = [t.name, t[0].name]
+        res_basis, res_keys = tree_basis(t)
+
+        assert_coo_allclose(exp_basis, res_basis)
+        self.assertListEqual(exp_keys, res_keys)
+
+    def test_tree_basis_unbalanced2(self):
+        tree = u"(d, (a,b)c);"
+
+        t = TreeNode.read([tree])
+
+        exp_basis = coo_matrix(np.array(
+            [
+                [-np.sqrt(2. / 3), np.sqrt(1. / 6), np.sqrt(1. / 6)],
+                [0, -np.sqrt(1. / 2), np.sqrt(1. / 2)]
+            ]
+        ))
+
+        exp_keys = [t.name, t[1].name]
+        res_basis, res_keys = tree_basis(t)
+        assert_coo_allclose(exp_basis, res_basis, atol=1e-7, rtol=1e-7)
+        self.assertListEqual(exp_keys, res_keys)
 
 
 class AncomTests(TestCase):
@@ -1164,5 +1234,76 @@ class AncomTests(TestCase):
             self.assertAlmostEqual(a, b)
 
 
+class TestVLR(TestCase):
+    def setUp(self):
+        self.mat = np.array([[1, 1, 2], [3, 5, 8], [13, 21, 55]])
+        self.mat_neg = np.array([[-1, 1, 2], [3, -5, 8], [13, 21, -55]])
+        self.mat_with_zero = np.array([[0, 1, 2], [3, 5, 8], [13, 21, 55]])
+
+    def test_vlr(self):
+        # No zeros
+        output = vlr(
+            x=self.mat[0],
+            y=self.mat[1],
+            ddof=1,
+            robust=False,
+        )
+        self.assertAlmostEqual(output, 0.0655828061998637)
+
+        # With zeros
+        output = vlr(
+            x=self.mat_with_zero[0],
+            y=self.mat_with_zero[1],
+            ddof=1,
+            robust=False,
+        )
+        assert np.isnan(output)
+
+        # assert raises error
+        with self.assertRaises(ValueError):
+            vlr(
+                x=self.mat_neg[0],
+                y=self.mat_neg[1],
+                ddof=1,
+                robust=False,
+            )
+
+    def test_robust_vlr(self):
+        # No zeros
+        output = vlr(
+            x=self.mat[0],
+            y=self.mat[1],
+            ddof=1,
+            robust=True,
+        )
+        self.assertAlmostEqual(output, 0.0655828061998637)
+
+        # With zeros
+        output = vlr(
+            x=self.mat_with_zero[0],
+            y=self.mat_with_zero[1],
+            ddof=1,
+            robust=True,
+        )
+        self.assertAlmostEqual(output, 0.024896522246558722)
+
+    def test_pairwise_vlr(self):
+
+        # No zeros
+        dism = pairwise_vlr(self.mat, ids=None, ddof=1, robust=False)
+        output = dism.condensed_form().sum()
+        self.assertAlmostEqual(output, 0.2857382286903922)
+
+        # With zeros
+        with self.assertRaises(DistanceMatrixError):
+            pairwise_vlr(self.mat_with_zero, ids=None, ddof=1, robust=False)
+
+        # no validation
+        dism = pairwise_vlr(self.mat, ids=None, ddof=1, robust=False,
+                            validate=False)
+        output = dism.data.sum() / 2
+        self.assertAlmostEqual(output, 0.2857382286903922)
+
+
 if __name__ == "__main__":
     main()
