@@ -3,7 +3,7 @@
 #
 # Distributed under the terms of the Modified BSD License.
 #
-# The full license is in the file COPYING.txt, distributed with this software.
+# The full license is in the file LICENSE.txt, distributed with this software.
 # ----------------------------------------------------------------------------
 
 import inspect
@@ -13,7 +13,7 @@ import sys
 import numpy as np
 import numpy.testing as npt
 import pandas.testing as pdt
-
+from scipy.spatial.distance import pdist
 from ._decorator import experimental
 
 
@@ -118,24 +118,26 @@ def assert_ordination_results_equal(left, right, ignore_method_names=False,
         npt.assert_equal(left.short_method_name, right.short_method_name)
         npt.assert_equal(left.long_method_name, right.long_method_name)
 
-    _assert_frame_equal(left.samples, right.samples,
-                        ignore_columns=ignore_axis_labels,
-                        ignore_directionality=ignore_directionality,
-                        decimal=decimal)
+    _assert_frame_dists_equal(left.samples, right.samples,
+                              ignore_columns=ignore_axis_labels,
+                              ignore_directionality=ignore_directionality,
+                              decimal=decimal)
 
-    _assert_frame_equal(left.features, right.features,
-                        ignore_columns=ignore_axis_labels,
-                        ignore_directionality=ignore_directionality,
-                        decimal=decimal)
-    _assert_frame_equal(left.biplot_scores, right.biplot_scores,
-                        ignore_columns=ignore_axis_labels,
-                        ignore_directionality=ignore_directionality,
-                        decimal=decimal)
+    _assert_frame_dists_equal(left.features, right.features,
+                              ignore_columns=ignore_axis_labels,
+                              ignore_directionality=ignore_directionality,
+                              decimal=decimal)
 
-    _assert_frame_equal(left.sample_constraints, right.sample_constraints,
-                        ignore_columns=ignore_axis_labels,
-                        ignore_directionality=ignore_directionality,
-                        decimal=decimal)
+    _assert_frame_dists_equal(left.biplot_scores, right.biplot_scores,
+                              ignore_columns=ignore_axis_labels,
+                              ignore_directionality=ignore_directionality,
+                              decimal=decimal)
+
+    _assert_frame_dists_equal(left.sample_constraints,
+                              right.sample_constraints,
+                              ignore_columns=ignore_axis_labels,
+                              ignore_directionality=ignore_directionality,
+                              decimal=decimal)
 
     _assert_series_equal(left.eigvals, right.eigvals, ignore_axis_labels,
                          decimal=decimal)
@@ -156,6 +158,24 @@ def _assert_series_equal(left_s, right_s, ignore_index=False, decimal=7):
             pdt.assert_index_equal(left_s.index, right_s.index)
 
 
+def _assert_frame_dists_equal(left_df, right_df, ignore_index=False,
+                              ignore_columns=False,
+                              ignore_directionality=False, decimal=7):
+    if left_df is None or right_df is None:
+        assert left_df is None and right_df is None
+    else:
+        left_values = left_df.values
+        right_values = right_df.values
+        left_dists = pdist(left_values)
+        right_dists = pdist(right_values)
+        npt.assert_almost_equal(left_dists, right_dists, decimal=decimal)
+
+        if not ignore_index:
+            pdt.assert_index_equal(left_df.index, right_df.index)
+        if not ignore_columns:
+            pdt.assert_index_equal(left_df.columns, right_df.columns)
+
+
 def _assert_frame_equal(left_df, right_df, ignore_index=False,
                         ignore_columns=False, ignore_directionality=False,
                         decimal=7):
@@ -246,7 +266,7 @@ def _normalize_signs(arr1, arr2):
 
 
 @experimental(as_of="0.4.0")
-def assert_data_frame_almost_equal(left, right):
+def assert_data_frame_almost_equal(left, right, rtol=1e-5):
     """Raise AssertionError if ``pd.DataFrame`` objects are not "almost equal".
 
     Wrapper of ``pd.util.testing.assert_frame_equal``. Floating point values
@@ -287,10 +307,10 @@ def assert_data_frame_almost_equal(left, right):
                            check_index_type=True,
                            check_column_type=True,
                            check_frame_type=True,
-                           check_less_precise=False,
                            check_names=True,
                            by_blocks=False,
-                           check_exact=False)
+                           check_exact=False,
+                           rtol=rtol)
     # this check ensures that empty DataFrames with different indices do not
     # compare equal. exact=True specifies that the type of the indices must be
     # exactly the same
@@ -304,7 +324,6 @@ def assert_series_almost_equal(left, right):
                             check_dtype=True,
                             check_index_type=True,
                             check_series_type=True,
-                            check_less_precise=False,
                             check_names=True,
                             check_exact=False,
                             check_datetimelike_compat=False,
