@@ -197,14 +197,14 @@ allow you to indicate ``anything`` as an option value, anything that is
 #
 # Distributed under the terms of the Modified BSD License.
 #
-# The full license is in the file COPYING.txt, distributed with this software.
+# The full license is in the file LICENSE.txt, distributed with this software.
 # ----------------------------------------------------------------------------
 
 import sys
 from copy import deepcopy
 from time import time
 from functools import update_wrapper
-from collections import Iterable
+from collections.abc import Iterable
 from types import MethodType
 
 from skbio.util._decorator import experimental
